@@ -24,7 +24,6 @@ require "logstash/util/time_value"
 require "i18n"
 
 module LogStash
-
   class Settings
 
     include LogStash::Util::SubstitutionVariables
@@ -77,7 +76,6 @@ module LogStash
       "queue.type",
     ]
 
-
     def initialize
       @settings = {}
       # Theses settings were loaded from the yaml file
@@ -125,7 +123,7 @@ module LogStash
       get_setting(setting_name).set?
     end
 
-    def clone
+    def clone(*args)
       get_subset(".*")
     end
     alias_method :dup, :clone
@@ -187,7 +185,7 @@ module LogStash
       @settings.values.each(&:reset)
     end
 
-    def from_yaml(yaml_path, file_name="logstash.yml")
+    def from_yaml(yaml_path, file_name = "logstash.yml")
       settings = read_yaml(::File.join(yaml_path, file_name))
       self.merge(deep_replace(flatten_hash(settings)), true)
       self
@@ -199,6 +197,13 @@ module LogStash
           callback.call(self)
         end
       end
+
+      # because we cannot rely on deprecation logger being wired up when the setters
+      # are initially used, we re-emit setter-related deprecations after all post-processing
+      # hooks have been activated (and therefore after logging has been configured)
+      @settings.each_value do |setting|
+        setting.observe_post_process if setting.respond_to?(:observe_post_process)
+      end
     end
 
     def on_post_process(&block)
@@ -225,12 +230,12 @@ module LogStash
       YAML.safe_load(IO.read(path)) || {}
     end
 
-    def flatten_hash(h,f="",g={})
+    def flatten_hash(h, f = "", g = {})
       return g.update({ f => h }) unless h.is_a? Hash
       if f.empty?
-        h.each { |k,r| flatten_hash(r,k,g) }
+        h.each { |k, r| flatten_hash(r, k, g) }
       else
-        h.each { |k,r| flatten_hash(r,"#{f}.#{k}",g) }
+        h.each { |k, r| flatten_hash(r, "#{f}.#{k}", g) }
       end
       g
     end
@@ -241,7 +246,7 @@ module LogStash
 
     attr_reader :name, :default
 
-    def initialize(name, klass, default=nil, strict=true, &validator_proc)
+    def initialize(name, klass, default = nil, strict = true, &validator_proc)
       @name = name
       unless klass.is_a?(Class)
         raise ArgumentError.new("Setting \"#{@name}\" must be initialized with a class (received #{klass})")
@@ -345,7 +350,7 @@ module LogStash
     end
 
     class Coercible < Setting
-      def initialize(name, klass, default=nil, strict=true, &validator_proc)
+      def initialize(name, klass, default = nil, strict = true, &validator_proc)
         @name = name
         unless klass.is_a?(Class)
           raise ArgumentError.new("Setting \"#{@name}\" must be initialized with a class (received #{klass})")
@@ -379,7 +384,7 @@ module LogStash
     ### Specific settings #####
 
     class Boolean < Coercible
-      def initialize(name, default, strict=true, &validator_proc)
+      def initialize(name, default, strict = true, &validator_proc)
         super(name, Object, default, strict, &validator_proc)
       end
 
@@ -396,7 +401,7 @@ module LogStash
     end
 
     class Numeric < Coercible
-      def initialize(name, default=nil, strict=true)
+      def initialize(name, default = nil, strict = true)
         super(name, ::Numeric, default, strict)
       end
 
@@ -416,7 +421,7 @@ module LogStash
     end
 
     class Integer < Coercible
-      def initialize(name, default=nil, strict=true)
+      def initialize(name, default = nil, strict = true)
         super(name, ::Integer, default, strict)
       end
 
@@ -434,7 +439,7 @@ module LogStash
     end
 
     class PositiveInteger < Integer
-      def initialize(name, default=nil, strict=true)
+      def initialize(name, default = nil, strict = true)
         super(name, default, strict) do |v|
           if v > 0
             true
@@ -448,7 +453,7 @@ module LogStash
     class Port < Integer
       VALID_PORT_RANGE = 1..65535
 
-      def initialize(name, default=nil, strict=true)
+      def initialize(name, default = nil, strict = true)
         super(name, default, strict) { |value| valid?(value) }
       end
 
@@ -460,8 +465,8 @@ module LogStash
     class PortRange < Coercible
       PORT_SEPARATOR = "-"
 
-      def initialize(name, default=nil, strict=true)
-        super(name, ::Range, default, strict=true) { |value| valid?(value) }
+      def initialize(name, default = nil, strict = true)
+        super(name, ::Range, default, strict = true) { |value| valid?(value) }
       end
 
       def valid?(range)
@@ -495,7 +500,7 @@ module LogStash
     end
 
     class Validator < Setting
-      def initialize(name, default=nil, strict=true, validator_class=nil)
+      def initialize(name, default = nil, strict = true, validator_class = nil)
         @validator_class = validator_class
         super(name, ::Object, default, strict)
       end
@@ -506,7 +511,7 @@ module LogStash
     end
 
     class String < Setting
-      def initialize(name, default=nil, strict=true, possible_strings=[])
+      def initialize(name, default = nil, strict = true, possible_strings = [])
         @possible_strings = possible_strings
         super(name, ::String, default, strict)
       end
@@ -527,7 +532,7 @@ module LogStash
     end
 
     class Password < Coercible
-      def initialize(name, default=nil, strict=true)
+      def initialize(name, default = nil, strict = true)
         super(name, LogStash::Util::Password, default, strict)
       end
 
@@ -594,7 +599,7 @@ module LogStash
     # then these options in the config file or command line will be all valid: "foo", true, false, "true", "false"
     #
     class CoercibleString < Coercible
-      def initialize(name, default=nil, strict=true, possible_strings=[], &validator_proc)
+      def initialize(name, default = nil, strict = true, possible_strings = [], &validator_proc)
         @possible_strings = possible_strings
         super(name, Object, default, strict, &validator_proc)
       end
@@ -612,9 +617,9 @@ module LogStash
     end
 
     class ExistingFilePath < Setting
-      def initialize(name, default=nil, strict=true)
+      def initialize(name, default = nil, strict = true)
         super(name, ::String, default, strict) do |file_path|
-          if !::File.exists?(file_path)
+          if !::File.exist?(file_path)
             raise ::ArgumentError.new("File \"#{file_path}\" must exist but was not found.")
           else
             true
@@ -624,7 +629,7 @@ module LogStash
     end
 
     class WritableDirectory < Setting
-      def initialize(name, default=nil, strict=false)
+      def initialize(name, default = nil, strict = false)
         super(name, ::String, default, strict)
       end
 
@@ -669,8 +674,8 @@ module LogStash
     end
 
     class Bytes < Coercible
-      def initialize(name, default=nil, strict=true)
-        super(name, ::Integer, default, strict=true) { |value| valid?(value) }
+      def initialize(name, default = nil, strict = true)
+        super(name, ::Integer, default, strict = true) { |value| valid?(value) }
       end
 
       def valid?(value)
@@ -698,7 +703,7 @@ module LogStash
     class TimeValue < Coercible
       include LogStash::Util::Loggable
 
-      def initialize(name, default, strict=true, &validator_proc)
+      def initialize(name, default, strict = true, &validator_proc)
         super(name, Util::TimeValue, default, strict, &validator_proc)
       end
 
@@ -716,7 +721,7 @@ module LogStash
     end
 
     class ArrayCoercible < Coercible
-      def initialize(name, klass, default, strict=true, &validator_proc)
+      def initialize(name, klass, default, strict = true, &validator_proc)
         @element_class = klass
         super(name, ::Array, default, strict, &validator_proc)
       end
@@ -744,7 +749,7 @@ module LogStash
     class SplittableStringArray < ArrayCoercible
       DEFAULT_TOKEN = ","
 
-      def initialize(name, klass, default, strict=true, tokenizer = DEFAULT_TOKEN, &validator_proc)
+      def initialize(name, klass, default, strict = true, tokenizer = DEFAULT_TOKEN, &validator_proc)
         @element_class = klass
         @token = tokenizer
         super(name, klass, default, strict, &validator_proc)
@@ -761,6 +766,26 @@ module LogStash
       end
     end
 
+    class StringArray < ArrayCoercible
+      def initialize(name, default, strict = true, possible_strings = [], &validator_proc)
+        @possible_strings = possible_strings
+        super(name, ::String, default, strict, &validator_proc)
+      end
+
+      protected
+
+      def validate(value)
+        super(value)
+        return unless @possible_strings&.any?
+
+        invalid_value = coerce(value).reject { |val| @possible_strings.include?(val) }
+        return unless invalid_value.any?
+
+        raise ArgumentError,
+          "Failed to validate the setting \"#{@name}\" value(s): #{invalid_value.inspect}. Valid options are: #{@possible_strings.inspect}"
+      end
+    end
+
     class Modules < Coercible
       def initialize(name, klass, default = nil)
         super(name, klass, default, false)
@@ -821,16 +846,14 @@ module LogStash
       end
 
       def set(value)
-        deprecation_logger.deprecated(I18n.t("logstash.settings.deprecation.set",
-                                             deprecated_alias: name,
-                                             canonical_name: canonical_proxy.name))
+        do_log_setter_deprecation
         super
       end
 
       def value
         logger.warn(I18n.t("logstash.settings.deprecation.queried",
-                           deprecated_alias: name,
-                           canonical_name: canonical_proxy.name))
+                           :deprecated_alias => name,
+                           :canonical_name => canonical_proxy.name))
         @canonical_proxy.value
       end
 
@@ -838,6 +861,18 @@ module LogStash
         # bypass deprecation warning
         wrapped.validate_value if set?
       end
+
+      def observe_post_process
+        do_log_setter_deprecation if set?
+      end
+
+      private
+
+      def do_log_setter_deprecation
+        deprecation_logger.deprecated(I18n.t("logstash.settings.deprecation.set",
+                                             :deprecated_alias => name,
+                                             :canonical_name => canonical_proxy.name))
+      end
     end
 
     ##
@@ -896,8 +931,8 @@ module LogStash
       def validate_value
         if deprecated_alias.set? && canonical_setting.set?
           fail(ArgumentError, I18n.t("logstash.settings.deprecation.ambiguous",
-                                     canonical_name: canonical_setting.name,
-                                     deprecated_alias: deprecated_alias.name))
+                                     :canonical_name => canonical_setting.name,
+                                     :deprecated_alias => deprecated_alias.name))
         end
 
         super
@@ -905,6 +940,5 @@ module LogStash
     end
   end
 
-
   SETTINGS = Settings.new
 end
